import { system, world, ItemStack, GameMode, EquipmentSlot } from '@minecraft/server';
// This array defines the tool items that will lose durability.
const toolTypeIds = [
    "tcc_copper_tools:copper_pickaxe",
    "tcc_copper_tools:copper_axe",
    "tcc_copper_tools:copper_shovel",
    "tcc_copper_tools:copper_hoe"
];
// This array defines the weapon items that will lose durability.
const weaponTypeIds = [
    "tcc_copper_tools:copper_sword"
];
// Defines hoes
const hoeIds = [
    'tcc_copper_tools:copper_hoe'
];
// Defines shovels
const shovelIds = [
    'tcc_copper_tools:copper_shovel'
];
// Defines axes
const axeIds = [
    'tcc_copper_tools:copper_axe'
];
world.afterEvents.playerBreakBlock.subscribe((evd)=>{
    const { player, itemStackBeforeBreak: itemUsed } = evd;
    if (!itemUsed || player.matches({
        gameMode: GameMode.creative
    })) return;
    if (toolTypeIds.includes(itemUsed.typeId) || weaponTypeIds.includes(itemUsed.typeId)) {
        const playerEquippableComp = player.getComponent("equippable");
        if (!playerEquippableComp) return;
        const itemEnchantmentComp = itemUsed.getComponent("minecraft:enchantable");
        const unbreakingLevel = itemEnchantmentComp?.getEnchantment("unbreaking")?.level ?? 0;
        const breakChance = 100 / (unbreakingLevel + 1);
        const randomizeChance = Math.random() * 100;
        if (breakChance < randomizeChance) return;
        const itemUsedDurabilityComp = itemUsed.getComponent("durability");
        if (!itemUsedDurabilityComp) return;
        let durabilityModifier = 0;
        if (toolTypeIds.includes(itemUsed.typeId)) {
            durabilityModifier = 1;
        } else {
            durabilityModifier = 2;
        }
        itemUsedDurabilityComp.damage += durabilityModifier;
        const maxDurability = itemUsedDurabilityComp.maxDurability;
        const currentDamage = itemUsedDurabilityComp.damage;
        if (currentDamage >= maxDurability) {
            player.playSound('random.break', {
                pitch: 1,
                location: player.location,
                volume: 1
            });
            playerEquippableComp.setEquipment(EquipmentSlot.Mainhand, new ItemStack('minecraft:air', 1));
        } else if (currentDamage < maxDurability) {
            playerEquippableComp.setEquipment(EquipmentSlot.Mainhand, itemUsed);
        }
    }
});
world.beforeEvents.itemUseOn.subscribe((event)=>{
    const { source: player, block, itemStack: itemUsed } = event;
    if (!itemUsed) return;
    if (shovelIds.includes(itemUsed.typeId)) {
        const playerEquippableComp = player.getComponent("equippable");
        if (!playerEquippableComp) return;
        if (shovelIds.includes(itemUsed.typeId)) {
            const blockAbove = block.dimension.getBlock(block.location).above(1);
            if (!blockAbove.isAir) return;
            if (block.permutation.matches('minecraft:dirt') || block.permutation.matches('minecraft:dirt_with_roots') || block.permutation.matches('minecraft:mycelium') || block.permutation.matches('minecraft:podzol')) {
                if (!block.permutation.matches('minecraft:grass_block')) {
                    system.run(function() {
                        block.setType("minecraft:grass_path");
                    });
                }
                system.run(function() {
                    player.playSound('use.grass', {
                        pitch: 1,
                        location: block.location,
                        volume: 0.5
                    });
                });
            } else return;
        }
        if (player.matches({
            gameMode: GameMode.creative
        })) return;
        const itemEnchantmentComp = itemUsed.getComponent("minecraft:enchantable");
        const unbreakingLevel = itemEnchantmentComp?.getEnchantment("unbreaking")?.level ?? 0;
        const breakChance = 100 / (unbreakingLevel + 1);
        const randomizeChance = Math.random() * 100;
        if (breakChance < randomizeChance) return;
        const itemUsedDurabilityComp = itemUsed.getComponent("durability");
        if (!itemUsedDurabilityComp) return;
        system.run(function() {
            itemUsedDurabilityComp.damage += 1;
            const maxDurability = itemUsedDurabilityComp.maxDurability;
            const currentDamage = itemUsedDurabilityComp.damage;
            if (currentDamage >= maxDurability) {
                system.run(()=>{
                    player.playSound('random.break', {
                        pitch: 1,
                        location: player.location,
                        volume: 1
                    });
                    playerEquippableComp.setEquipment(EquipmentSlot.Mainhand, new ItemStack('minecraft:air', 1));
                });
            } else if (currentDamage < maxDurability) {
                system.run(()=>{
                    playerEquippableComp.setEquipment(EquipmentSlot.Mainhand, itemUsed);
                });
            }
        });
    }
});
world.afterEvents.itemUseOn.subscribe((event)=>{
    const { source: player, block, itemStack: itemUsed } = event;
    if (!itemUsed) return;
    if (hoeIds.includes(itemUsed.typeId) || shovelIds.includes(itemUsed.typeId) || axeIds.includes(itemUsed.typeId)) {
        const playerEquippableComp = player.getComponent("equippable");
        if (!playerEquippableComp) return;
        // Hoes
        if (hoeIds.includes(itemUsed.typeId)) {
            const blockAbove = block.dimension.getBlock(block.location).above(1);
            if (!blockAbove.isAir) return;
            if (block.permutation.hasTag("dirt") || block.permutation.hasTag("grass")) {
                player.playSound('use.gravel', {
                    pitch: 1,
                    location: block.location,
                    volume: 1
                });
            } else return;
        }
        // Shovels
        if (shovelIds.includes(itemUsed.typeId)) {
            const blockAbove = block.dimension.getBlock(block.location).above(1);
            if (!blockAbove.isAir) return;
            if (block.permutation.hasTag("dirt") || block.permutation.hasTag("grass")) {
                player.playSound('use.grass', {
                    pitch: 1,
                    location: block.location,
                    volume: 0.5
                });
            } else return;
        }
        // Axes
        if (axeIds.includes(itemUsed.typeId)) {
            if (block.permutation.matches('minecraft:stripped_cherry_log')) player.playSound('step.cherry_wood', {
                pitch: 1,
                location: block.location,
                volume: 1
            });
            else if (block.permutation.matches('minecraft:stripped_bamboo_block')) player.playSound('step.bamboo_wood', {
                pitch: 1,
                location: block.location,
                volume: 1
            });
            else if (block.permutation.matches('minecraft:stripped_crimson_stem') || block.permutation.matches('minecraft:stripped_warped_stem')) player.playSound('use.stem', {
                pitch: 1,
                location: block.location,
                volume: 1
            });
            else if (block.typeId.includes("stripped")) player.playSound('use.wood', {
                pitch: 1,
                location: block.location,
                volume: 1
            });
            else return;
        }
        if (player.matches({
            gameMode: GameMode.creative
        })) return;
        const itemEnchantmentComp = itemUsed.getComponent("minecraft:enchantable");
        const unbreakingLevel = itemEnchantmentComp?.getEnchantment("unbreaking")?.level ?? 0;
        const breakChance = 100 / (unbreakingLevel + 1);
        const randomizeChance = Math.random() * 100;
        if (breakChance < randomizeChance) return;
        const itemUsedDurabilityComp = itemUsed.getComponent("durability");
        if (!itemUsedDurabilityComp) return;
        itemUsedDurabilityComp.damage += 1;
        const maxDurability = itemUsedDurabilityComp.maxDurability;
        const currentDamage = itemUsedDurabilityComp.damage;
        if (currentDamage >= maxDurability) {
            player.playSound('random.break', {
                pitch: 1,
                location: player.location,
                volume: 1
            });
            playerEquippableComp.setEquipment(EquipmentSlot.Mainhand, new ItemStack('minecraft:air', 1));
        } else if (currentDamage < maxDurability) {
            playerEquippableComp.setEquipment(EquipmentSlot.Mainhand, itemUsed);
        }
    }
});
